@@ -1,9 +1,10 @@
-# $Id: RobotUA.pm,v 1.11 1997/12/02 13:22:52 aas Exp $
+# $Id: RobotUA.pm,v 1.14 1998/11/19 21:45:00 aas Exp $
 
 package LWP::RobotUA;
 
 require LWP::UserAgent;
 @ISA = qw(LWP::UserAgent);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/);
 
 require WWW::RobotRules;
 require HTTP::Request;
@@ -46,11 +47,7 @@ special agent will make sure you are nice.
 =head1 METHODS
 
 The LWP::RobotUA is a sub-class of LWP::UserAgent and implements the
-same methods.  The use_alarm() method also desides whether we will
-wait if a request is tried too early (if true), or will return an error
-response (if false).
-
-In addition these methods are provided:
+same methods. In addition the following methods are provided:
 
 =over 4
 
@@ -199,7 +196,6 @@ sub simple_request
 	# fetch "robots.txt"
 	my $robot_url = $request->url->clone;
 	$robot_url->path("robots.txt");
-	$robot_url->params(undef);
 	$robot_url->query(undef);
 	LWP::Debug::debug("Requesting $robot_url");
 
@@ -228,7 +224,7 @@ sub simple_request
 	  &HTTP::Status::RC_FORBIDDEN, 'Forbidden by robots.txt';
     }
 
-    my $netloc = $request->url->netloc;
+    my $netloc = $request->url->host_port;
     my $wait = $self->host_wait($netloc);
 
     if ($wait) {
